# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

@author: ilgar
"""
import re


def empty(val):
    """
    Checks if value is empty.
    All unknown data types considered as empty values.
    @return: bool
    """
    if val == None:
        return True

    if isinstance(val,str) and len(val) > 0:
        return False

    return True


def get_domain(email):
    """
    Returns domain part of the email or None if invalid email format.
    @param email: email
    @return: str
    """
    match = re.search('^[^@]*?@([^@]+?)$', email)

    if match == None:
        return None

    return match.group(1)



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Account info samples.

@author: ilgar
"""
from ConfigParser import ConfigParser
from pyzimbra import soap
from pyzimbra.z.client import ZimbraClient
from test import pconstant
import logging
import sys


def load_properties():
    properties = "../client-local.properties"
    cfg = ConfigParser()
    cfg.read(properties)

    p = {}
    p[pconstant.ADMIN_HOSTNAME] = cfg.get(pconstant.ADMIN, pconstant.HOST)
    p[pconstant.ADMIN_ACCOUNT_NAME] = cfg.get(pconstant.ADMIN, pconstant.USERNAME)
    p[pconstant.ADMIN_PASSWORD] = cfg.get(pconstant.ADMIN, pconstant.PASSWORD)

    p[pconstant.DOMAIN] = cfg.get(pconstant.DOMAIN, pconstant.NAME)
    p[pconstant.HOSTNAME] = cfg.get(pconstant.DOMAIN, pconstant.HOST)
    p[pconstant.DOMAINS] = {p[pconstant.DOMAIN]:
                            cfg.get(pconstant.DOMAIN, pconstant.KEY)}

    p[pconstant.PROXY_SCHEME] = cfg.get(pconstant.PROXY, pconstant.SCHEME)
    p[pconstant.PROXY_HOSTNAME] = cfg.get(pconstant.PROXY, pconstant.HOST)
    p[pconstant.PROXY_PORT] = cfg.get(pconstant.PROXY, pconstant.PORT)
    p[pconstant.PROXY_USERNAME] = cfg.get(pconstant.PROXY, pconstant.USERNAME)
    p[pconstant.PROXY_PASSWORD] = cfg.get(pconstant.PROXY, pconstant.PASSWORD)

    p[pconstant.USERNAME] = cfg.get(pconstant.AUTH, pconstant.USERNAME)
    p[pconstant.ACCOUNT_NAME] = '%s@%s' % (p[pconstant.USERNAME], p[pconstant.DOMAIN])
    p[pconstant.PASSWORD] = cfg.get(pconstant.AUTH, pconstant.PASSWORD)

    return p


def init_client(f):
    def call():
        logging.basicConfig(stream=sys.stdout,level=logging.DEBUG)

        p = load_properties()

        zclient = ZimbraClient(soap.soap_url(p[pconstant.HOSTNAME]))
        zclient.authenticate(p[pconstant.ACCOUNT_NAME], p[pconstant.PASSWORD])

        return f(p, zclient)
    return call


def init_proxied_client(f):
    def call():
        logging.basicConfig(stream=sys.stdout,level=logging.DEBUG)

        p = load_properties()

        proxy_url = soap.proxy_url(p[pconstant.PROXY_HOSTNAME],
                                   p[pconstant.PROXY_USERNAME],
                                   p[pconstant.PROXY_PASSWORD],
                                   p[pconstant.PROXY_PORT],
                                   p[pconstant.PROXY_SCHEME])
    
        zclient = ZimbraClient(soap.soap_url(p[pconstant.HOSTNAME]),
                               proxy_url=proxy_url)
        zclient.authenticate(p[pconstant.ACCOUNT_NAME], p[pconstant.PASSWORD])

        return f(p, zclient)
    return call



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Zimbra client related methods and classes.

@author: ilgar
"""
import abc
import traceback


class ZObject(object):
    """
    Base class for data objects.
    """
    __metaclass__ = abc.ABCMeta

    # ------------------------------------------------------------------ unbound
    @abc.abstractmethod
    def parse(self, res):
        """
        Parses xml structure.
        @param res: xml response from zimbra
        """


class ZimbraClientException(Exception):
    """
    Zimbra client exception.
    """
    # --------------------------------------------------------------- properties
    message = property(lambda self: self._message, 
                       lambda self, v: setattr(self, '_message', v))
    tracebacks = property(lambda self: self._tracebacks,
                          lambda self, v: setattr(self, '_tracebacks', v))

    # -------------------------------------------------------------------- bound
    def __init__(self, message, cause = None):
        Exception.__init__(self, message)

        self._message = message
        self.__cause__ = cause

        self.tracebacks = []
        if cause != None:
            if isinstance(cause, ZimbraClientException):
                self.tracebacks = cause.tracebacks
        list = traceback.format_exc().split('\n')[1:]
        self.tracebacks.insert(0, '\n'.join(list))


    def __str__(self):
        return self.__unicode__()


    def __unicode__(self):
        return self.message


    # ------------------------------------------------------------------ unbound
    def print_trace(self):
        """
        Prints stack trace for current exceptions chain.
        """
        traceback.print_exc()
        for tb in self.tracebacks:
            print tb,
        print ''


class ZimbraClientTransport(object):
    """
    Transport base class. 
    """
    __metaclass__ = abc.ABCMeta

    # --------------------------------------------------------------- properties
    soap_url = property(lambda self: self._soap_url,
                        lambda self, v: setattr(self, '_soap_url', v))
    proxy_url = property(lambda self: self._proxy_url,
                         lambda self, v: setattr(self, '_proxy_url', v))

    # -------------------------------------------------------------------- bound
    def __init__(self):
        self.soap_url = None
        self.proxy_url = None

    # ------------------------------------------------------------------ unbound
    @abc.abstractmethod
    def invoke(self, ns, request_name, params, auth_token, simplify):
        """
        Invokes zimbra request.
        @param ns: namespace of the request method
        @param request_name: name of the request method
        @param params: parameters to pass to method call
        @param auth_token: authentication token to use for session
        @param simplify: True to return python object, False to return xml struct
        @return: zimbra response
        """
        if auth_token == None:
            raise ZimbraClientException('Invalid auth token')



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

@author: ilgar
"""

CONTEXT = 'context'

AuthRequest = 'AuthRequest'
AuthResponse = 'AuthResponse'
ChangePasswordRequest = 'ChangePasswordRequest'
ChangePasswordResponse = 'ChangePasswordResponse'
GetAccountRequest = 'GetAccountRequest'
GetAccountResponse = 'GetAccountResponse'
GetAccountInfoRequest = 'GetAccountInfoRequest'
GetAccountInfoResponse = 'GetAccountInfoResponse'
GetInfoRequest = 'GetInfoRequest'
GetInfoResponse = 'GetInfoResponse'

E_AUTH_TOKEN = 'authToken'
E_SESSION_ID = 'sessionId'
E_ACCOUNT = 'account'
E_PREAUTH = 'preauth'
E_PASSWORD = 'password'
E_OLD_PASSWORD = 'oldPassword'
E_NAME = 'name'

A_BY = 'by'
A_EXPIRES = 'expires'
A_ID = 'id'
A_SECTIONS = 'sections'
A_TIMESTAMP = 'timestamp'

V_ID = 'id'
V_NAME = 'name'
V_MBOX = 'mbox'



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Authentication related methods and classes.

@author: ilgar
"""
from pyzimbra import util
from pyzimbra.base import ZimbraClientTransport, ZimbraClientException
import abc


class AuthException(ZimbraClientException):
    """
    Authentication exception.
    """
    # -------------------------------------------------------------------- bound
    def __init__(self, message, cause = None):
        ZimbraClientException.__init__(self, message, cause)


class AuthToken(object):
    """
    AuthToken holds current authentication token and sessions id.
    """
    # --------------------------------------------------------------- properties
    account_name = property(lambda self: self._account_name,
                            lambda self, v: setattr(self, '_account_name', v))
    token = property(lambda self: self._token,
                     lambda self, v: setattr(self, '_token', v))
    session_id = property(lambda self: self._session_id,
                          lambda self, v: setattr(self, '_session_id', v))


    # -------------------------------------------------------------------- bound
    def __init__(self):
        self.account_name = None
        self.token = None
        self.session_id = None


    def __str__(self):
        return self.__unicode__()


    def __unicode__(self):
        return ('account_name: %s; token: %s; session_id: %s'
                % (self.account_name, self.token, self.session_id))


class Authenticator(object):
    """
    Authenticator provides methods to authenticate using username/password 
    as a user or administrator or using domain key.
    """
    __metaclass__ = abc.ABCMeta

    # --------------------------------------------------------------- properties
    domains = property(lambda self: self._domains,
                       lambda self, v: setattr(self, '_domains', v))

    # -------------------------------------------------------------------- bound
    def __init__(self):
        self.domains = {}

    # ------------------------------------------------------------------ unbound
    @abc.abstractmethod
    def authenticate_admin(self, transport, account_name, password):
        """
        Authenticates administrator using username and password.
        @param transport: transport to use for method calls
        @param account_name: account name
        @param password: account password
        @return: AuthToken if authentication succeeded
        @raise AuthException: if authentication fails
        """

    @abc.abstractmethod
    def authenticate(self, transport, account_name, password):
        """
        Authenticates account, if no password given tries to pre-authenticate.
        @param transport: transport to use for method calls
        @param account_name: account name
        @param password: account password
        @return: AuthToken if authentication succeeded
        @raise AuthException: if authentication fails
        """
        if not isinstance(transport, ZimbraClientTransport):
            raise ZimbraClientException('Invalid transport')

        if util.empty(account_name):
            raise AuthException('Empty account name')



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
#
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
#
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Zimbra non-privileged client.

@author: ilgar
"""
from pyzimbra import sconstant, zconstant
from pyzimbra.z.zobject.account import AccountInfo
from pyzimbra.zclient import ZimbraSoapClient
import SOAPpy


class ZimbraClient(ZimbraSoapClient):
    """
    Zimbra non-privileged client.
    """
    # ------------------------------------------------------------------ unbound
    def authenticate(self, account_name, password):
        """
        Authenticates zimbra account.
        @param account_name: account email address
        @param password: account password
        @raise AuthException: if authentication fails
        @raise SoapException: if soap communication fails
        """
        self.auth_token = self.authenticator.authenticate(self.transport,
                                                          account_name,
                                                          password)


    def change_password(self, current_password, new_password):
        """
        Changes account password.
        @param current_password: current password
        @param new_password: new password
        """
        attrs = {sconstant.A_BY: sconstant.V_NAME}
        account = SOAPpy.Types.stringType(data=self.auth_token.account_name,
                                          attrs=attrs)

        params = {sconstant.E_ACCOUNT: account,
                  sconstant.E_OLD_PASSWORD: current_password,
                  sconstant.E_PASSWORD: new_password}

        self.invoke(zconstant.NS_ZIMBRA_ACC_URL,
                    sconstant.ChangePasswordRequest,
                    params)


    def get_account_info(self):
        """
        Gets account info.
        @return: AccountInfo
        """
        attrs = {sconstant.A_BY: sconstant.V_NAME}
        account = SOAPpy.Types.stringType(data=self.auth_token.account_name,
                                          attrs=attrs)

        params = {sconstant.E_ACCOUNT: account}

        res = self.invoke(zconstant.NS_ZIMBRA_ACC_URL,
                          sconstant.GetAccountInfoRequest,
                          params)

        info = AccountInfo()
        info.parse(res)

        return info


    def get_info(self, params={}):
        """
        Gets mailbox info.
        @param params: params to retrieve
        @return: AccountInfo
        """
        res = self.invoke(zconstant.NS_ZIMBRA_ACC_URL,
                          sconstant.GetInfoRequest,
                          params)

        return res


    def get_search_folder_request(self, attrs={}, methodattrs=None):
        l = SOAPpy.Types.untypedType(data='', attrs=attrs)
        params = {'folder': l}

        res = self.invoke(zconstant.NS_ZIMBRA_MAIL_URL,
                          'GetFolderRequest',
                          params,
                          methodattrs=methodattrs)

        return res



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
#
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
#
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

SOAPPy related methods and classes.

@author: ilgar
"""
from pyzimbra import zconstant, util
from pyzimbra.soap import SoapException
import SOAPpy
import logging
import urllib2
import xml.sax


class ZimbraSOAPParser(SOAPpy.SOAPParser):
    """
    No need to keep track of hrefs for zimbra.
    Ugliest hack ever: just empty list of ref ids every time.
    Could not find another workaround.

    SOAPpy deletes node's id attribute. So we back it up before deletion.

    Add basic rules for parsing nested tags attributes. They are skipped by
    SOAPpy otherwise.
    """
    # -------------------------------------------------------------------- bound
    def __init__(self, rules = None):
        if rules is None:
            rules = {
                # this rule makes nested e and m tags inside c returned by
                # SearchResponse for conversations be parsed with
                # attributes, otherwise SOAPpy skips attributes
                'SearchResponse': {
                    'c': {
                        'e': {},
                        'm': {}
                    },
                    'task': {
                      'or': {
                        'inst': {
                          'fr': {}
                        }
                      }
                    },
                },
                'GetConvResponse': {
                    'c': {
                        'm': {
                            'e': {}
                        }
                    }
                },
                'CreateTaskResponse': {
                },
                'GetMsgResponse': {
                    'm': {
                        's': {},
                        'd': {},
                        'rev': {},
                        'id': {},
                        'inv': {
                            'type': {},
                            'comp': {},
                        },
                    },
                },
                'GetFolderResponse': {
                    'folder': {
                        'id': {},
                        'name': {},
                        'absFolderPath': {},
                        'uuid': {},
                        'luuid': {},
                        'rev': {},
                        'i4next': {},
                        'i4ms': {},
                        'activesyncdisabled': {},
                        'l': {},
                        's': {},
                        'ms': {},
                        'n': {},
                        'folder': {
                            'folder': {
                                'folder': {},
                            },
                        },
                    },
                },
            }

        SOAPpy.SOAPParser.__init__(self, rules)


    # ------------------------------------------------------------------ unbound
    def endElementNS(self, name, qname):
        # try to back up id attribute before it'll be removed by SOAPpy parser
        frame = self._stack[-1]
        if frame.attrs.has_key((None, 'id')):
            frame.attrs[(None, '_orig_id')] = frame.attrs[(None, 'id')]

        self._ids = {}
        SOAPpy.SOAPParser.endElementNS(self, name, qname)


def parseSOAP(xml_str, rules = None):
    """
    Replacement for SOAPpy._parseSOAP method to spoof SOAPParser.
    """
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO

    parser = xml.sax.make_parser()
    t = ZimbraSOAPParser(rules = rules)
    parser.setContentHandler(t)
    e = xml.sax.handler.ErrorHandler()
    parser.setErrorHandler(e)

    inpsrc = xml.sax.xmlreader.InputSource()
    inpsrc.setByteStream(StringIO(xml_str))

    # turn on namespace mangeling
    parser.setFeature(xml.sax.handler.feature_namespaces,1)

    try:
        parser.parse(inpsrc)
    except xml.sax.SAXParseException, e:
        parser._parser = None
        raise e

    return t


class SoapHttpTransport(SOAPpy.Client.HTTPTransport):
    """
    Http transport using urllib2, with support for proxy authentication and more.
    """
    # --------------------------------------------------------------- properties
    transport = property(lambda self: self._transport,
                         lambda self, v: setattr(self, '_transport', v))


    # -------------------------------------------------------------------- bound
    def __init__(self):
        self.log = logging.getLogger(__name__)


    # ------------------------------------------------------------------ unbound
    def call(self, addr, data, namespace, soapaction = None, encoding = None,
        http_proxy = None, config = SOAPpy.Config, timeout = None):

        if not isinstance(addr, SOAPpy.Client.SOAPAddress):
            addr = SOAPpy.Client.SOAPAddress(addr, config)

        url = addr.proto + "://" + addr.host + addr.path

        headers = {'User-Agent': zconstant.USER_AGENT}
        request = urllib2.Request(url, data, headers)

        self.log.debug('Request url: %s' % url)
        self.log.debug('Request headers')
        self.log.debug(request.headers)
        self.log.debug('Request data')
        self.log.debug(data)

        try:
            opener = self.build_opener()
            response = opener.open(request)
            data = response.read()

            self.log.debug('Response headers')
            self.log.debug(response.headers)
            self.log.debug('Response data')
            self.log.debug(data)

        except urllib2.URLError as exc:
            raise self.init_soap_exception(exc)

        # get the new namespace
        if namespace is None:
            new_ns = None
        else:
            new_ns = self.getNS(namespace, data)

        return data, new_ns


    def build_opener(self):
        """
        Builds url opener, initializing proxy.
        @return: OpenerDirector
        """
        http_handler = urllib2.HTTPHandler()

        if util.empty(self.transport.proxy_url):
            return urllib2.build_opener(http_handler)

        proxy_handler = urllib2.ProxyHandler(
            {self.transport.proxy_url[:4]: self.transport.proxy_url})

        return urllib2.build_opener(http_handler, proxy_handler)


    def init_soap_exception(self, exc):
        """
        Initializes exception based on soap error response.
        @param exc: URLError
        @return: SoapException
        """
        if not isinstance(exc, urllib2.HTTPError):
            return SoapException(unicode(exc), exc)

        if isinstance(exc, urllib2.HTTPError):
            try:
                data = exc.read()
                self.log.debug(data)

                t = SOAPpy.Parser.parseSOAP(data)
                message = '%s:%s' % (t.Fault.faultcode, t.Fault.faultstring)
                e = SoapException(message, exc)
                e.code = t.Fault.detail.Error.Code
                e.trace = t.Fault.detail.Error.Trace
                return e
            except:
                return SoapException(unicode(exc), exc)

        return SoapException(exc.reason, exc)



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
#
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
#
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Soap related methods and classes.

@author: ilgar
"""
from pyzimbra import zconstant, sconstant, util
from pyzimbra.auth import AuthException, AuthToken, Authenticator
from pyzimbra.soap import SoapException
from time import time
import SOAPpy
import hashlib
import hmac
import logging

class SoapAuthenticator(Authenticator):
    """
    Soap authenticator.
    """
    # --------------------------------------------------------------- properties

    # -------------------------------------------------------------------- bound
    def __init__(self):
        Authenticator.__init__(self)
        self.log = logging.getLogger(__name__)

    # ------------------------------------------------------------------ unbound
    def authenticate_admin(self, transport, account_name, password):
        """
        Authenticates administrator using username and password.
        """
        Authenticator.authenticate_admin(self, transport, account_name, password)

        auth_token = AuthToken()
        auth_token.account_name = account_name

        params = {sconstant.E_NAME: account_name,
                  sconstant.E_PASSWORD: password}

        self.log.debug('Authenticating admin %s' % account_name)
        try:
            res = transport.invoke(zconstant.NS_ZIMBRA_ADMIN_URL,
                                   sconstant.AuthRequest,
                                   params,
                                   auth_token)
        except SoapException as exc:
            raise AuthException(unicode(exc), exc)

        auth_token.token = res.authToken
        auth_token.session_id = res.sessionId

        self.log.info('Authenticated admin %s, session id %s'
                      % (account_name, auth_token.session_id))

        return auth_token


    def authenticate(self, transport, account_name, password=None):
        """
        Authenticates account using soap method.
        """
        Authenticator.authenticate(self, transport, account_name, password)

        if password == None:
            return self.pre_auth(transport, account_name)
        else:
            return self.auth(transport, account_name, password)


    def auth(self, transport, account_name, password):
        """
        Authenticates using username and password.
        """
        auth_token = AuthToken()
        auth_token.account_name = account_name

        attrs = {sconstant.A_BY: sconstant.V_NAME}
        account = SOAPpy.Types.stringType(data=account_name, attrs=attrs)

        params = {sconstant.E_ACCOUNT: account,
                  sconstant.E_PASSWORD: password}

        self.log.debug('Authenticating account %s' % account_name)
        try:
            res = transport.invoke(zconstant.NS_ZIMBRA_ACC_URL,
                                   sconstant.AuthRequest,
                                   params,
                                   auth_token)
        except SoapException as exc:
            raise AuthException(unicode(exc), exc)

        if type(res) is tuple:
            auth_token.token = res[0].authToken
        else:
            auth_token.token = res.authToken

        if hasattr(res, 'sessionId'):
            auth_token.session_id = res.sessionId

        self.log.info('Authenticated account %s, session id %s'
                      % (account_name, auth_token.session_id))

        return auth_token


    def pre_auth(self, transport, account_name):
        """
        Authenticates using username and domain key.
        """
        auth_token = AuthToken()
        auth_token.account_name = account_name

        domain = util.get_domain(account_name)
        if domain == None:
            raise AuthException('Invalid auth token account')

        if domain in self.domains:
            domain_key = self.domains[domain]
        else:
            domain_key = None

        if domain_key == None:
            raise AuthException('Invalid domain key for domain %s' % domain)

        self.log.debug('Initialized domain key for account %s'
                      % account_name)

        expires = 0
        timestamp = int(time() * 1000)
        pak = hmac.new(domain_key, '%s|%s|%s|%s' %
                       (account_name, sconstant.E_NAME, expires, timestamp),
                       hashlib.sha1).hexdigest()

        attrs = {sconstant.A_BY: sconstant.V_NAME}
        account = SOAPpy.Types.stringType(data=account_name, attrs=attrs)

        attrs = {sconstant.A_TIMESTAMP: timestamp, sconstant.A_EXPIRES: expires}
        preauth = SOAPpy.Types.stringType(data=pak,
                                          name=sconstant.E_PREAUTH,
                                          attrs=attrs)

        params = {sconstant.E_ACCOUNT: account,
                  sconstant.E_PREAUTH: preauth}

        self.log.debug('Authenticating account %s using domain key'
                       % account_name)
        try:
            res = transport.invoke(zconstant.NS_ZIMBRA_ACC_URL,
                                   sconstant.AuthRequest,
                                   params,
                                   auth_token)
        except SoapException as exc:
            raise AuthException(unicode(exc), exc)

        auth_token.token = res.authToken
        auth_token.session_id = res.sessionId

        self.log.info('Authenticated account %s, session id %s'
                      % (account_name, auth_token.session_id))

        return auth_token



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################.

@author: ilgar
"""
from client.util import init_client


@init_client
def run(p, zclient):

    info = zclient.get_account_info()
    print info


if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Authentication samples.

@author: ilgar
"""
from client.util import load_properties
from pyzimbra import soap
from pyzimbra.base import ZimbraClientException
from pyzimbra.soap_auth import SoapAuthenticator
from pyzimbra.soap_transport import SoapTransport
from test import pconstant
import logging
import sys


def authenticate():
    p = load_properties()

    transport = SoapTransport()
    transport.soap_url = soap.soap_url(p[pconstant.HOSTNAME])

    auth = SoapAuthenticator()
    auth_token = auth.authenticate(transport,
                                   p[pconstant.ACCOUNT_NAME],
                                   p[pconstant.PASSWORD])

    print auth_token


def pre_authenticate():
    p = load_properties()

    transport = SoapTransport()
    transport.debug = 1
    transport.soap_url = soap.soap_url(p[pconstant.HOSTNAME])

    auth = SoapAuthenticator()
    auth.domains = p[pconstant.DOMAINS]
    auth_token = auth.authenticate(transport, p[pconstant.ACCOUNT_NAME])

    print auth_token


if __name__ == '__main__':
    logging.basicConfig(stream=sys.stdout,level=logging.DEBUG)

    try:
        authenticate()
    except ZimbraClientException, e:
        e.print_trace()

    try:
        pre_authenticate()
    except ZimbraClientException, e:
        e.print_trace()



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Soap related methods and classes.

@author: ilgar
"""
from pyzimbra import util, zconstant
from pyzimbra.base import ZimbraClientException
import urllib2


def admin_soap_url(hostname):
    """
    @return: absolute zimbra administrative soap endpoint url
    @raise SoapException: if hostname is empty
    """
    if util.empty(hostname):
        raise SoapException('Empty hostname')

    return 'https://%s%s' % (hostname, zconstant.SOAP_ADMIN_URL)


def soap_url(hostname):
    """
    @return: absolute zimbra soap endpoint url
    @raise SoapException: if hostname is empty
    """
    if util.empty(hostname):
        raise SoapException('Empty hostname')

    return 'http://%s%s' % (hostname, zconstant.SOAP_URL)


def proxy_url(hostname, username=None, password=None, port=0, scheme='http'):
    """
    @return: absolute proxy url
    @raise SoapException: if scheme or hostname is empty
    """
    if util.empty(hostname):
        raise SoapException('Empty hostname')

    if util.empty(scheme):
        raise SoapException('Empty scheme')

    hostport = hostname
    if port > 0 and (scheme != 'http' or port != 80):
        hostport = '%s:%s' % (hostname, port)

    userpass = ''
    if not util.empty(username):
        userpass = '%s:%s@' % (username, password)

    url = '%s://%s%s' % (scheme, userpass, hostport)

    return url


class SoapException(ZimbraClientException):
    """
    Soap exception.
    """
    # --------------------------------------------------------------- properties
    code = property(lambda self: self._code, 
                    lambda self, v: setattr(self, '_code', v))
    trace = property(lambda self: self._trace, 
                     lambda self, v: setattr(self, '_trace', v))

    # -------------------------------------------------------------------- bound
    def __init__(self, message, cause = None):
        ZimbraClientException.__init__(self, message, cause)

        self.code = None
        self.trace = None

        if cause != None and isinstance(cause, urllib2.HTTPError):
            self.code = cause.code


    def __unicode__(self):
        return '%s (%s:%s)' % (self.message, self.code, self.trace)



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
#
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
#
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Soap related methods and classes.

@author: ilgar
"""
from pyzimbra import zconstant, sconstant
from pyzimbra.base import ZimbraClientTransport
from pyzimbra.soap_soappy import parseSOAP, SoapHttpTransport
import SOAPpy
import logging


class SoapTransport(ZimbraClientTransport):
    """
    Soap transport.
    """
    # --------------------------------------------------------------- properties
    http_transport = SoapHttpTransport()


    # -------------------------------------------------------------------- bound
    def __init__(self):
        ZimbraClientTransport.__init__(self)
        self.http_transport.transport = self
        self.log = logging.getLogger(__name__)


    # ------------------------------------------------------------------ unbound
    def invoke(self, ns, request_name, params, auth_token, simplify=False,
               methodattrs=None):
        """
        Invokes zimbra soap request.
        """
        ZimbraClientTransport.invoke(self,
                                     ns,
                                     request_name,
                                     params,
                                     auth_token,
                                     simplify)

        headers = SOAPpy.Types.headerType()


        if auth_token.token != None:
            data={sconstant.E_AUTH_TOKEN: auth_token.token,
                  sconstant.E_SESSION_ID: auth_token.session_id}
            context = SOAPpy.Types.structType(data=data, name=sconstant.CONTEXT)
            context._validURIs = []
            context._ns = (zconstant.SOAP_DEFAULT_PREFIX, zconstant.NS_ZIMBRA_URL)
            headers.context = context

        proxy = SOAPpy.SOAPProxy(self.soap_url,
                                 ns,
                                 header=headers,
                                 noroot=1,
                                 methodattrs=methodattrs,
                                 simplify_objects=simplify)
        proxy.config.debug = self.log.isEnabledFor(logging.DEBUG)
        proxy.config.strictNamespaces = 0
        proxy.config.buildWithNamespacePrefix = 0
        proxy.config.returnAllAttrs = True
        proxy.config.dumpSOAPOut = False
        proxy.config.dumpSOAPIn = False
        proxy.transport = self.http_transport

        _parseSOAP = SOAPpy.Parser._parseSOAP
        SOAPpy.Parser._parseSOAP = parseSOAP
        try:
            m = proxy.__getattr__(request_name)
            return m.__call__(**params)
        finally:
            SOAPpy.Parser._parseSOAP = _parseSOAP



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Account info samples.

@author: ilgar
"""
from client.util import load_properties
from pyzimbra import sconstant, zconstant, soap
from pyzimbra.z.admin import ZimbraAdmin
from test import pconstant
import SOAPpy
import logging
import sys


def get_account():
    p = load_properties()

    zclient = ZimbraAdmin(soap.admin_soap_url(p[pconstant.ADMIN_HOSTNAME]))
    zclient.authenticate(p[pconstant.ADMIN_ACCOUNT_NAME],
                         p[pconstant.ADMIN_PASSWORD])

    attrs = {sconstant.A_BY: sconstant.V_NAME}
    account = SOAPpy.Types.stringType(data=p[pconstant.ACCOUNT_NAME], attrs=attrs)

    params = {sconstant.E_ACCOUNT: account}
    res = zclient.invoke(zconstant.NS_ZIMBRA_ADMIN_URL,
                         sconstant.GetAccountRequest,
                         params)

    print res


if __name__ == '__main__':
    logging.basicConfig(stream=sys.stdout,level=logging.DEBUG)

    get_account()



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Zimbra client related methods and classes.

@author: ilgar
"""
from pyzimbra import util
from pyzimbra.auth import AuthException
from pyzimbra.base import ZimbraClientException
from pyzimbra.soap_auth import SoapAuthenticator
from pyzimbra.soap_transport import SoapTransport
import abc


class ZimbraSoapClient(object):
    """
    Zimbra client main class.
    """
    __metaclass__ = abc.ABCMeta

    # --------------------------------------------------------------- properties
    transport = property(lambda self: self._transport,
                         lambda self, v: setattr(self, '_transport', v))
    authenticator = property(lambda self: self._authenticator,
                             lambda self, v: setattr(self, '_authenticator', v))
    auth_token = property(lambda self: self._auth_token,
                          lambda self, v: setattr(self, '_auth_token', v))


    # -------------------------------------------------------------------- bound
    def __init__(self, soap_url, domains={}, proxy_url=None):
        self.transport = SoapTransport()
        self.transport.soap_url = soap_url
        
        if proxy_url != None:
            self.transport.proxy_url = proxy_url

        self.authenticator = SoapAuthenticator()
        self.authenticator.domains = domains
        
        self.auth_token = None


    # ------------------------------------------------------------------ unbound
    def invoke(self, ns, request_name, params={}, simplify=False, 
               methodattrs=None):
        """
        Invokes zimbra method using established authentication session.
        @param req: zimbra request
        @parm params: request params
        @param simplify: True to return python object, False to return xml struct
        @param methodattrs: attributes of the method
        @return: zimbra response
        @raise AuthException: if authentication fails
        @raise SoapException: wrapped server exception
        """
        if self.auth_token == None:
            raise AuthException('Unable to invoke zimbra method')

        if util.empty(request_name):
            raise ZimbraClientException('Invalid request')

        return self.transport.invoke(ns,
                                     request_name,
                                     params,
                                     self.auth_token,
                                     simplify,
                                     methodattrs)


    @abc.abstractmethod
    def authenticate(self, account_name, password):
        """
        Authenticates zimbra account.
        @param account_name: account email address
        @param password: account password
        @raise AuthException: if authentication fails
        @raise SoapException: if soap communication fails
        """



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
#
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
#
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

@author: ilgar
"""

__version__ = '0.1'
USER_AGENT = "pyzimbra/%s" % __version__

SOAP_URL = '/service/soap'
SOAP_ADMIN_URL = '/service/admin/soap'

SOAP_DEFAULT_PREFIX = 'ns1'

NS_SOAP_URL = 'http://www.w3.org/2003/05/soap-envelope'
NS_SOAP_PREFIX = 'soap'
NS_SOAP_MAP = {NS_SOAP_PREFIX: NS_SOAP_URL}
NS_SOAP = '{%s}' % NS_SOAP_URL

NS_ZIMBRA_URL = 'urn:zimbra'
NS_ZIMBRA_MAP = {None: NS_ZIMBRA_URL}
NS_ZIMBRA = '{%s}' % NS_ZIMBRA_URL

NS_ZIMBRA_ACC_URL = 'urn:zimbraAccount'
NS_ZIMBRA_ACC_MAP = {None: NS_ZIMBRA_ACC_URL}
NS_ZIMBRA_ACC = '{%s}' % NS_ZIMBRA_ACC_URL

NS_ZIMBRA_ADMIN_URL = 'urn:zimbraAdmin'
NS_ZIMBRA_ACC_MAP = {None: NS_ZIMBRA_ADMIN_URL}
NS_ZIMBRA_ACC = '{%s}' % NS_ZIMBRA_ADMIN_URL

NS_ZIMBRA_MAIL_URL = 'urn:zimbraMail'
NS_ZIMBRA_MAIL_MAP = {None: NS_ZIMBRA_ACC_URL}
NS_ZIMBRA_MAIL = '{%s}' % NS_ZIMBRA_ACC_URL




# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# 
# E-mail: pyzimbra@lab.az
# Website: http://github.com/ilgarm/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Zimbra privileged client.

@author: ilgar
"""
from pyzimbra import sconstant, zconstant
from pyzimbra.zclient import ZimbraSoapClient


class ZimbraAdmin(ZimbraSoapClient):
    """
    Zimbra non-privileged client.
    """
    # ------------------------------------------------------------------ unbound
    def authenticate(self, account_name, password):
        """
        Authenticates zimbra account.
        @param account_name: account email address
        @param password: account password
        @raise AuthException: if authentication fails
        @raise SoapException: if soap communication fails
        """
        self.auth_token = self.authenticator.authenticate_admin(self.transport,
                                                                account_name,
                                                                password)


    def get_account(self):
        """
        Gets account.
        @return: Account
        """


    def change_password(self, account, password):
        """
        Changes account password.
        @param account: account to change password for
        @param password: new password
        """


    def get_info(self, account, params={}):
        """
        Gets account info.
        @param account: account to get info for
        @param params: parameters to retrieve
        @return: AccountInfo
        """
        res = self.invoke(zconstant.NS_ZIMBRA_ADMIN_URL,
                          sconstant.GetInfoRequest,
                          params)

        return res



# -*- coding: utf-8 -*-
"""
################################################################################
# Copyright (c) 2010, Ilgar Mashayev
# Copyright (c) 2014, VNC AG
# 
# E-mail: support@vnc.biz
# Website: http://github.com/vnc-biz/pyzimbra
################################################################################
# This file is part of pyzimbra.
# 
# Pyzimbra is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyzimbra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyzimbra.  If not, see <http://www.gnu.org/licenses/>.
################################################################################

Distutil setup.

@author: ilgar
"""

from setuptools import setup, find_packages

files = ["*", "z/*"]

setup(name = 'pyzimbra',
    version = '0.1',
    license = 'LGPL',
    description = 'Zimbra Python Client',
    long_description=open("README.rst").read(),
    author = 'Ilgar Mashayev',
    author_email = 'pyzimbra@lab.az',
    url = 'http://github.com/ilgarm/pyzimbra',
    packages = ['pyzimbra','pyzimbra/z', 'pyzimbra/z/zobject/'],
    package_data = {'package' : files },
    install_requires=[
        'SOAPpy',
    ],
)
